// code written by Bart Huyskens  25/10/2021  for www.e2cre8.be
// hardware: Brainbox AVR (www.e2cre8.be >> Brainbox AVR)(This is a rugged Arduino Leonardo) ; HC06 Bluetooth module www.e2cre8.be ; Android Smartphone ; I2C LCD 4x20 www.e2cre8.be
// software: Arduino IDE 1.6.11 ; Appinventor
// Principle of communication:
// The app sends data to the uC in this format: "L:xxx;R:xxx;" where xxx is data between 0-255.
// The speed is determined by the TimerInterval of Timer1 in Appinventor and this needs to be identical to the variable "TimerIntervalAppinventor" in this program
// Speeds up to 100msec seem to be OK, but if you do not need this speed, use 1000msec
// The uC waits checks for incoming data at half of "TimerIntervalAppinventor" -time to be sure to not miss to much
// The uC send 3 random values to the app, every 2x "TimerIntervalAppinventor". This needs to be double the time for appinventor to be ready to receive the next data

#define TimerIntervalAppinventor 500      // this value needs to be equal to the value of the timer in Appinventor
                                          // tested & works pretty stable at 100msec, but if you don't need the speed, set it at 500msec both here and in Appinventor

#define tx 1          // pin number for TX pin
#define rx 0          // pin number for RX pin

int NrOfCycles;       // to slow down the transmission of data
int RValue = 0;
int LValue = 0;
int RValueOld = 0;
int LValueOld = 0;
int i = 0;
int z = 0;
bool connectionOK = 0;

char inSerial[13] = "";  // Array of characters (is not a string)- used to read the received data
int LValueArray[4];
int RValueArray[4];

String myString;        // used to do string manipulations on the received data

void setup()
{
  Serial.begin(9600);             //For Serial monitor
  Serial1.begin(9600);            // comms with HC06 - baudrate 9600
  pinMode(tx, OUTPUT);
  pinMode(rx, INPUT);

  while (Serial1.available())         //clear serial RX buffer
  {
    char t = Serial1.read();
  }
}

void loop()
{
  // Set up 3 variables to be sent
  int VAL1 = analogRead(A0);                  // TX data 3 potmeters at A0, A2 & A4
  int VAL2 = analogRead(A2);
  int VAL3 = analogRead(A4);

  delay(TimerIntervalAppinventor / 2);     // timerInterval of Appinventor devided by 2, to be ready to receive all incoming data

  NrOfCycles = NrOfCycles + 1;             // transmission of data can only execute after 4 receive cycles

  //***RECEIVING DATA ***//
  if (Serial1.available() > 0)             // If receiving serial data from HC06
  {
    connectionOK = 1;                     // only start sending data after reception of first data -else uC crashes
    i = 0;
    while (Serial1.available())           // while receiving serial data from HC06
    {
      inSerial[i] = Serial1.read();       // load incoming bytes in array
      i++;                                // increment i
    }
  }

  //***TRANSMITTING DATA ****//
  if ((NrOfCycles >= 4) && connectionOK)       // transmission of data can only execute after 4 receive cycles for Appinventor to be ready to receive data
  {
    Serial1.print(VAL1);            // send varialble VAL1 via Serial1 to HC06
    Serial1.print('|');             // send '|' character - appinventor uses this to split the data
    Serial1.print(VAL2);
    Serial1.print('|');
    Serial1.print(VAL3);
    Serial1.print('|');
    NrOfCycles = 0;
  }


  //****DECODING & FILTERING OF INCOMING DATA***//
    myString = String(inSerial);          // convert array to string to be able to use string manipulation instructions (indexOf...)

  int Lpos = myString.indexOf('L');                           // find the position of character 'L' in the string
  if (Lpos == -1)                           // if Lpos = -1 then Lpos is not found
  {
    while (Serial1.available())         //clear serial RX buffer
    {
      char t = Serial1.read();
    }
  }
  else
  {
    int m = 0;
    LValueOld = LValue;
    LValue = 0; // reset L value
    while ((myString.charAt(Lpos + 2 + m)) != ';')                      //convert characters into RValue numbers untill character ';'
    {                                                                   // start 2 positions after character 'R'
      LValue = LValue * 10 + ((myString.charAt(Lpos + 2 + m)) - 48);    //char '0' has ascii value 48
      m++;
    }
        if ((LValue > 255) || (LValue < 0)) LValue = LValueOld;   // protection from corrupt data
        
        // Accept data only if last 3 values were identical - this way glitches are filtered out
        LValueArray[3] = LValueArray[2];
        LValueArray[2] = LValueArray[1];
        LValueArray[1] = LValueArray[0];
        LValueArray[0] = LValue;
          if ((LValueArray[2] == LValueArray[1]) && (LValueArray[1] == LValueArray[0]))  LValue = LValueArray[0];
          else LValue =  LValueArray[3];

    int Rpos = myString.indexOf('R');                           // find the position of character 'R' in the string
    m = 0;
    RValueOld = RValue;
    RValue = 0;                                                 // Reset R Value
    while ((myString.charAt(Rpos + 2 + m)) != ';')              //convert characters into RValue numbers untill character ';'
    { // start 2 positions after character 'R'
      RValue = RValue * 10 + ((myString.charAt(Rpos + 2 + m)) - 48); //char '0' has ascii value 48
      m++;
    }
        if ((RValue > 255) || (RValue < 0)) RValue = RValueOld;   // protection from corrupt data
        
        // Accept data only if last 3 values were identical
        RValueArray[3] = RValueArray[2];
        RValueArray[2] = RValueArray[1];
        RValueArray[1] = RValueArray[0];
        RValueArray[0] = RValue;
          if ((RValueArray[2] == RValueArray[1]) && (RValueArray[1] == RValueArray[0])) RValue = RValueArray[0]; 
          else  RValue =  RValueArray[3];

    // print op Seriele monitor
    if (connectionOK != 1)
    {
      Serial.println("NOT CONNECTED TO HC06");      
    }
    else
    {
      Serial.print(myString);               // Laat ontvangen string zien
      Serial.print("\t");                   // Serial monitor - TAB
      Serial.print(LValue);                 // Serial monitor Value of L
      Serial.print("\t");                   // Serial monitor - TAB
      Serial.print(RValue);                 // Serial monitor Value of R
      Serial.print("\t");                   // Serial monitor - TAB
      Serial.print(VAL1);                   // Laat verzonden string zien
      Serial.print('|');                    // send '|' character - appinventor uses this to split the data
      Serial.print(VAL2);
      Serial.print('|');
      Serial.print(VAL3);
      Serial.print('|');
      Serial.print("\t");                   // Serial monitor - TAB
      Serial.println("CONNECTED TO HC06");   
    }
  }
}
